@@ -60,7 +60,8 @@ class SettingsDialog : public MasterDialog {
         LayoutPage,
         WebCompanionPage,
         WebApplicationPage,
-        ExperimentalPage
+        ExperimentalPage,
+        AiPage
     };
 
     explicit SettingsDialog(int page = 0, QWidget *parent = 0);
@@ -323,6 +324,20 @@ class SettingsDialog : public MasterDialog {
     void on_nextcloudDeckStackTreeWidget_currentItemChanged(QTreeWidgetItem *current,
                                                             QTreeWidgetItem *previous);
 
+    void on_groqApiKeyWebButton_clicked();
+
+    void on_openAiApiKeyWebButton_clicked();
+
+    void on_showStatusBarNotePathCheckBox_toggled(bool checked);
+
+    void on_groqApiTestButton_clicked();
+
+    void on_openAiApiTestButton_clicked();
+
+    void on_groqApiKeyLineEdit_textChanged(const QString &arg1);
+
+    void on_openAiApiKeyLineEdit_textChanged(const QString &arg1);
+
    private:
     Ui::SettingsDialog *ui;
     QStatusBar *noteFolderRemotePathTreeStatusBar;
@@ -431,6 +446,10 @@ class SettingsDialog : public MasterDialog {
     void resetOKLabelData();
 
     void loadNextcloudDeckStackTreeWidget();
+
+    void buildAiScriptingTreeWidget();
+
+    void runAiApiTest(QString backend, QString model, QString apiKey = QString());
 };
 
 #endif    // SETTINGSDIALOG_H
